### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectCopiedEvent, IObjectRemovedEvent

# import local interfaces
from ztfy.sequence.interfaces import ISequentialIntIds, ISequentialIdTarget, ISequentialIdInfo

# import Zope3 packages
from zope.component import adapter, queryUtility
from zope.interface import implements
from zope.intid import IntIds
from zope.schema.fieldproperty import FieldProperty

# import local packages


class SequentialIntIds(IntIds):
    """Sequential IDs utility"""

    implements(ISequentialIntIds)

    prefix = FieldProperty(ISequentialIntIds['prefix'])
    hex_oid_length = FieldProperty(ISequentialIntIds['hex_oid_length'])

    _lastId = 0

    def _generateId(self):
        self._lastId += 1
        return self._lastId

    def register(self, ob):
        if not ISequentialIdTarget.providedBy(ob):
            return None
        return super(SequentialIntIds, self).register(ob)

    def generateHexId(self, obj, oid):
        return ('%%s%%s%%.%dx' % self.hex_oid_length) % (self.prefix or '',
                                                         getattr(obj, 'prefix', ''),
                                                         oid)


@adapter(ISequentialIdTarget, IObjectAddedEvent)
def handleNewSequentialIdTarget(obj, event):
    """Set unique ID for each added object"""
    utility = queryUtility(ISequentialIntIds, getattr(obj, 'sequence_name', ''))
    if utility is not None:
        info = ISequentialIdInfo(obj)
        if not info.oid:
            oid = info.oid = utility.register(obj)
            info.hex_oid = utility.generateHexId(obj, oid)


@adapter(ISequentialIdTarget, IObjectCopiedEvent)
def handleCopiedSequentialIdTarget(obj, event):
    """Reset unique ID when an object is copied"""
    info = ISequentialIdInfo(obj)
    info.oid = None
    info.hex_oid = None


@adapter(ISequentialIdTarget, IObjectRemovedEvent)
def handleRemovedSequentialIdTarget(obj, event):
    """Unregister object when it is removed"""
    utility = queryUtility(ISequentialIntIds, getattr(obj, 'sequence_name', ''))
    if (utility is not None) and (utility.queryId(obj) is not None):
        utility.unregister(obj)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

# import standard packages

# import Zope3 interfaces

# import local interfaces
from ztfy.sequence.interfaces import ISequentialIntIds
from ztfy.skin.interfaces import IDefaultView, IPropertiesMenuTarget
from ztfy.skin.layer import IZTFYBackLayer

# import Zope3 packages
from z3c.form import field
from zope.component import adapts
from zope.interface import implements, Interface
from zope.traversing.browser import absoluteURL

# import local packages
from ztfy.skin.form import EditForm


class SequentialIntIdsUtilityDefaultViewAdapter(object):
    """Sequential ID utility default view adapter"""

    adapts(ISequentialIntIds, IZTFYBackLayer, Interface)
    implements(IDefaultView)

    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.view = view

    @property
    def viewname(self):
        return '@@properties.html'

    def getAbsoluteURL(self):
        return '%s/%s' % (absoluteURL(self.context, self.request), self.viewname)


class SequentialIntIdsUtilityEditForm(EditForm):
    """Sequential ID utility edit form"""

    implements(IPropertiesMenuTarget)

    fields = field.Fields(ISequentialIntIds)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages
from persistent import Persistent

# import Zope3 interfaces
from zope.annotation.interfaces import IAnnotations

# import local interfaces
from ztfy.sequence.interfaces import ISequentialIdInfo, ISequentialIdTarget

# import Zope3 packages
from zope.component import adapter
from zope.interface import implements, implementer
from zope.schema.fieldproperty import FieldProperty

# import local packages


SEQUENCE_INFO_KEY = 'ztfy.sequence'

class SequentialIdInfo(Persistent):
    """Sequential ID info"""

    implements(ISequentialIdInfo)

    oid = FieldProperty(ISequentialIdInfo['oid'])
    hex_oid = FieldProperty(ISequentialIdInfo['hex_oid'])


@adapter(ISequentialIdTarget)
@implementer(ISequentialIdInfo)
def SequentialIdInfoFactory(context):
    annotations = IAnnotations(context)
    info = annotations.get(SEQUENCE_INFO_KEY)
    if info is None:
        info = annotations[SEQUENCE_INFO_KEY] = SequentialIdInfo()
    return info



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2012 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from zope.annotation.interfaces import IAttributeAnnotatable

# import local interfaces

# import Zope3 packages
from zope.interface import Interface
from zope.schema import TextLine, Int

# import local packages

from ztfy.sequence import _


class ISequentialIntIds(Interface):
    """Sequential IntIds utility interface"""

    prefix = TextLine(title=_("Hexadecimal prefix"),
                      description=_("Prefix used to generate hexadecimal ID"),
                      required=False,
                      max_length=10)

    hex_oid_length = Int(title=_("Hexadecimal ID length"),
                         description=_("Full length of hexadecimal ID, not including prefix"),
                         min=0,
                         max=20,
                         default=10)

    def generateHexId(self, obj, oid):
        """Generate an hexadecimal ID for the given sequence"""


class ISequentialIdInfo(Interface):
    """Sequential ID info interface"""

    oid = Int(title=_("Sequential ID"),
              required=False)

    hex_oid = TextLine(title=_("Sequential ID in hexadecimal form"),
                       required=False)


class ISequentialIdTarget(IAttributeAnnotatable):
    """Marker interface used to identify classes which have to get a unique sequential ID"""

    sequence_name = TextLine(title=_("Sequence name"),
                             description=_("Name of sequence utility used to get unique IDs"),
                             required=False)

    prefix = TextLine(title=_("Hexadecimal prefix"),
                      description=_("""Prefix used to generate hexadecimal ID, placed after utility prefix.
                                       Generally defined at class level."""),
                      required=False)
